-- localisations
local L = AceLibrary("AceLocale-2.2"):new("ShakedownMSBT")
L:RegisterTranslations("enUS", function() return {
    ["Mik's Scrolling Battle Text"] = true,
} end)

-- constants
local FREQUENCY = 1
local DRAG_WIDTH = 80
local DRAG_HEIGHT = 20

-- members
Addon.prototype.dragFrame = nil
Addon.prototype.deltaX = 0
Addon.prototype.deltaY = 0

-- create the class
local AceOO = AceLibrary("AceOO-2.0")
Addon = AceOO.Class(ShakedownAddonBase, "AceEvent-2.0")

-- constructor
function Addon.prototype:init()
    Addon.super.prototype.init(self, "MikScrollingBattleText", L["Mik's Scrolling Battle Text"])
end

function Addon.prototype:Activate()

    if not Addon.super.prototype.Activate(self) then return end
    
    if not self.dragFrame then
        self.dragFrame = {}
    end
    
    -- create the movers
    for i, v in ipairs(MSBTProfiles.CurrentProfile.ScrollAreaSettings) do
        if #self.dragFrame < i then
            self.dragFrame[i] = self:CreateDragFrame("MSBTDragFrame"..i)
        end
        self:ShowDragFrame(self.dragFrame[i], MSBTProfiles.CurrentProfile.ScrollAreaSettings[i].OffsetX, MSBTProfiles.CurrentProfile.ScrollAreaSettings[i].OffsetY)
    end
    
    -- calculate the co-ord delta between that which we get from GetPoint and the centre aligned co-ords that SCT works with
    self.dragFrame[1]:StartMoving()
    self.dragFrame[1]:StopMovingOrSizing()
    local _, _, _, x, y = self.dragFrame[1]:GetPoint()
    self.deltaX = MSBTProfiles.CurrentProfile.ScrollAreaSettings[1].OffsetX - x
    self.deltaY = MSBTProfiles.CurrentProfile.ScrollAreaSettings[1].OffsetY - y
    
    -- display something now
    self:Display()
    
    -- display a message every second
    self:ScheduleRepeatingEvent("MSBTShakedownDisplay", self.Display, FREQUENCY, self)

end

function Addon.prototype:Deactivate()

    Addon.super.prototype.Deactivate(self)

    -- stop display scheduling
    self:CancelScheduledEvent("MSBTShakedownDisplay")
    
    -- hide the movers
    for i, v in ipairs(MSBTProfiles.CurrentProfile.ScrollAreaSettings) do
        self.dragFrame[i]:Hide()
    end
end

function Addon.prototype:Display()

    for k, v in pairs(MSBTProfiles.CurrentProfile.ScrollAreaSettings) do
        MikSBT.DisplayMessage("MSBT "..v.Name, v.Name)
    end
    
end

function Addon.prototype:CreateDragFrame(name)
    local frame = CreateFrame("Frame", name, UIParent)
    
    frame:SetBackdrop({ bgFile = self.backdropMayDrag, })
    frame:SetWidth(DRAG_WIDTH)
    frame:SetHeight(DRAG_HEIGHT)
    
    frame:EnableMouse(true)
    frame:SetMovable(true)
	frame:SetScript("OnDragStart", function(this) self:DragStart(this) end)
	frame:SetScript("OnDragStop", function(this) self:DragStop(this) end)
	frame:RegisterForDrag("LeftButton")
    
    local label = frame:CreateFontString("dragFrameLabel", "BORDER")
    label:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
    label:SetFont(NumberFontNormal:GetFont(), 12, "OUTLINE")
    label:SetText("MSBT")
    
    return frame

end

function Addon.prototype:ShowDragFrame(frame, x, y)
    
    frame:ClearAllPoints()
    frame:SetPoint("CENTER", UIParent, "CENTER", x, y)
    frame:Show()
    
end

function Addon.prototype:DragStart(this)

    this.isDragging = true
	this:StartMoving()
end

function Addon.prototype:DragStop(this)

    this:StopMovingOrSizing()
    this.isDragging = false
    
    local _, _, _, x, y = this:GetPoint()
    local _, _, i = string.find(this:GetName(),"(%d)")
    if i then
        i = tonumber(i)
        local settings = MSBTProfiles.CurrentProfile.ScrollAreaSettings[i]
        if settings then
            settings.OffsetX = x + self.deltaX
            settings.OffsetY = y + self.deltaY
        end
    end
end

-- create the instance of this class; parent's constructor will handle registration with the main shakedown code
local thisAddon = Addon:new()